/**
Name:		ESP8266_NTPClass_test.ino
Created:	16.02.2016 23:10:56
Author:		Yuri Gradov aka IOPA4
ToDo:		
*/

/******************************************************************
ESP8266 with PCD8544 display
== Connection ==

USB TTL     Huzzah      Nokia 5110  Description
            ESP8266     PCD8544

            GND         GND         Ground
            3V          VCC         3.3V from Huzzah to display
            14          CLK         Output from ESP SPI clock
            13          DIN         Output from ESP SPI MOSI to display data input
            12          D/C         Output from display data/command to ESP
            #5          CS          Output from ESP to chip select/enable display
            #4          RST         Output from ESP to reset display
                        LED         3.3V to turn backlight on
******************************************************************/

// ESP8266 Software SPI (slower updates, more flexible pin options):
// pin 14 - Serial clock out (SCLK)
// pin 13 - Serial data out (DIN)
// pin 12 - Data/Command select (D/C)
// pin 5 - LCD chip select (CS)
// pin 4 - LCD reset (RST)

#include <Arduino.h>
#include <ESP8266WiFi.h>
#include <ESP8266HTTPClient.h>
#include "NTP_Client.h"
#include <SPI.h>
#include <Adafruit_GFX.h>
#include <Adafruit_PCD8544.h>
#include <Wire.h>
#include "TimeClient.h"
#include <ArduinoJson.h>

const char* ssid     = "************"; //имя вашей WiFi сети
const char* password = "********";     //и пароль доступа
const String server  = "api.openweathermap.org"; //зарегистрироваться на этом сайте
const String appid   = "********************************"; //и получить бесплатный ключ на получение погоды
const String lat     = "69.07"; //ваша широта
const String lon     = "33.42"; //ваша долгота
const String url     = "http://" + server + "/data/2.5/weather?lat=" + lat + "&lon=" + lon + "&units=metric&appid=" + appid;

bool first = 1;
String T = "";
String V = "";
String znak = "+";
unsigned int rumb;
String msg = "  ";
const char* vind[8] = {"С","СВ","В","ЮВ","Ю","ЮЗ","З","СЗ"};


String httpData;

struct weather_structure {
  unsigned int id;
  const char* main;
  const char* icon;
  const char* descript;
  float temp;
  float pressure;
  byte  humidity;
  float speed;
  float deg;
};
weather_structure weather;

WiFiClient client;

Adafruit_PCD8544 display = Adafruit_PCD8544(14, 13, 12, 5, 4);
CNTPTime ntpTime;


String IntToStr(int N)
{
  if (N<10) return "0" + String(N);
  else return String(N);
}

void PrintXY(char x, char y, String str) {
    display.setCursor(x,y);
    display.print(str);  
}

void setup() {

	Serial.begin(115200);

//	Serial.println();
//	Serial.println();
//	Serial.println();

  display.begin();
  display.cp437(true);
  display.setContrast(50);
  display.setTextColor(BLACK);
  display.setTextSize(1);
  display.clearDisplay();
  PrintXY(0, 0,(utf8rus("Старт WiFi! \r\n")));
  display.display();
 
	WiFi.begin(ssid, password);
	delay(2000);
	while (WiFi.status() != WL_CONNECTED) {
		 delay(200);
	}
   display.println(utf8rus("Cоединение"));
   display.println(ssid);
	 display.print(utf8rus("установлено!"));
   display.display();
   delay(2000);
	 ntpTime.Init();
}

void loop() {
	static int prevSec = 0;
  if (millis()%300 == 0) ntpTime.getTime(); //запрашиваем время каждые 300 милисекунд

	//обновляем дисплей каждую секунду
	if (prevSec != ntpTime.Second) {
    String H = IntToStr(ntpTime.Hour);
    String M = IntToStr(ntpTime.Minute);
    String S = IntToStr(ntpTime.Second);
    String D = IntToStr(ntpTime.Day) + "/" + IntToStr(ntpTime.Month) + "/" + IntToStr(ntpTime.Year);
    //запрашиваем погоду каждые 15 минут
    if ((ntpTime.Minute%15 == 0) and ((ntpTime.Second == 0)) or first)
    {
      httpRequest();
      parseData();
      if (weather.temp>=0) znak = "+"; else znak = '-';
      rumb = round((weather.deg + 22.5)/45.0);
      if (rumb==8) rumb=0;
      msg = vind[rumb];
      V = utf8rus(msg)+ " " + String(round(weather.speed)) + " m/c";
      T =  znak + String(round(weather.temp))+" "+ String(round(weather.humidity))+"% "+String(round(weather.pressure * 0.75))+" mm";
    }
	  display.clearDisplay();
    display.setTextSize(1);
    PrintXY(0, 0, T); 
    PrintXY(14, 8, V);  
    PrintXY(10, 17, D);
    display.setTextSize(3);
    PrintXY(1, 26, H);
    PrintXY(47, 26, M);
    display.setTextSize(1);
    PrintXY(35, 34, S);
    display.display();
		prevSec = ntpTime.Second;
    first = 0;
  }
}

bool httpRequest() {
  HTTPClient wclient;
  bool find = false;
  wclient.begin(url);
  int httpCode = wclient.GET();

  if (httpCode > 0) {
//    Serial.printf("successfully, code: %d\n", httpCode);
    if (httpCode == HTTP_CODE_OK) {
      httpData = wclient.getString();
      if (httpData.indexOf(F("\"main\":{\"temp\":")) > -1) {
//        lastConnectionTime = millis();
        find = true;
      }
//      else Serial.println("Failed, json string is not found");
    }
  }
//  else Serial.printf("failed, error: %s\n", wclient.errorToString(httpCode).c_str());

//  postingInterval = find ? 600L * 1000L : 60L * 1000L;
  wclient.end();

  return find;
}

bool parseData() {
//  Serial.println(httpData);

  DynamicJsonBuffer jsonBuffer;
  JsonObject& root = jsonBuffer.parseObject(httpData);

  if (!root.success()) {
//    Serial.println("Json parsing failed!");
    return false;
  }

  weather.id       = root["weather"][0]["id"];
  weather.main     = root["weather"][0]["main"];
  weather.descript = root["weather"][0]["description"];
  weather.icon     = root["weather"][0]["icon"];
  weather.temp     = root["main"]["temp"];
  weather.humidity = root["main"]["humidity"];
  weather.pressure = root["main"]["pressure"];
  weather.speed    = root["wind"]["speed"];
  weather.deg      = root["wind"]["deg"];

  httpData = "";
  return true;
}
