#include "NTP_Client.h"
#include <TimeLib.h>

void CNTPTime::getNTPTime()
{
	uint8_t packetBuffer[NTP_PACKET_SIZE]; //buffer to hold incoming & outgoing packets

	const long ntpFirstFourBytes = 0xEC0600E3; // NTP request header
	int pktLen;								   // received packet length

	if (!udpInited) return;					   // Fail if WiFiUdp.begin() could not init a socket

	bool isName = (serversName[0] != nullptr);

	for (int i = 0; i < 3 && (isName && serversName[0] != nullptr || !isName && ntpServers[i] != nullptr); i++) {
		// Clear received data from possible stray received packets
		Udp.flush();
		// Send an NTP request
		if (!isName && !Udp.beginPacket(*ntpServers[i], 123) || (isName && !Udp.beginPacket(serversName[i], 123))) continue; // 123 is the NTP port

		if (!(Udp.write((byte *)&ntpFirstFourBytes, NTP_PACKET_SIZE) == NTP_PACKET_SIZE
			&& Udp.endPacket()))
			continue;				// sending request failed

									// Wait for response; check every pollIntv ms up to maxPoll times
		const int pollIntv = 150;		// poll every this many ms
		const byte maxPoll = 15;		// poll up to this many times
		for (int j = 0; j < maxPoll; j++) {
			if ((pktLen = Udp.parsePacket()) == NTP_PACKET_SIZE) {
				if (i > 0) {
					if (!isName) {
						IPAddress *swapAddr = ntpServers[0];
						ntpServers[0] = ntpServers[i];
						ntpServers[i] = swapAddr;
					}
					else {
						char *swapName = serversName[0];
						serversName[0] = serversName[i];
						serversName[i] = swapName;
					}
	//				Serial.printf("\n...swap %i...\n", i);
				}
				break;
			}
			delay(pollIntv);
		}
		if (pktLen == NTP_PACKET_SIZE) break;
	}
	if (pktLen != NTP_PACKET_SIZE) return;
	Udp.read(packetBuffer, NTP_PACKET_SIZE);  // read packet into the buffer
											  // convert four bytes starting at location 40 to a long integer
	secsSince1970 = (unsigned long)packetBuffer[40] << 24;
	secsSince1970 |= (unsigned long)packetBuffer[41] << 16;
	secsSince1970 |= (unsigned long)packetBuffer[42] << 8;
	secsSince1970 |= (unsigned long)packetBuffer[43];
	setTime(SECS_SINCE_1900);
	regMillis = millis();
	fillDataFields();
}

inline void CNTPTime::fillDataFields()
{
	Day = day();
	Month = month();
	Year = year();
	Hour = hour();
	Minute = minute();
	Second = second();
}

CNTPTime::CNTPTime(unsigned long ntpInterval, const uint8_t timeZone_, const uint16_t ntpPort_, IPAddress * ntpServer1, IPAddress * ntpServer2, IPAddress * ntpServer3) : timeZone(timeZone_), ntpPort(ntpPort_)
{
	this->ntpInterval = ntpInterval;
	if (ntpServer1 == nullptr) ntpServers[0] = new IPAddress(88, 147, 254, 232); else ntpServers[0] = ntpServer1;
	if (ntpServer2 == nullptr) ntpServers[1] = new IPAddress(193, 34, 152, 1); else ntpServers[1] = ntpServer2;
	if (ntpServer3 == nullptr) ntpServers[2] = new IPAddress(40, 76, 58, 209); else ntpServers[2] = ntpServer3;
}

void CNTPTime::Init()
{
	udpInited = Udp.begin(ntpPort);
}

bool CNTPTime::getTime()
{
	const unsigned long t = millis();
	if (t - regMillis >= ntpInterval || regMillis == 0) {
		getNTPTime();
	}
	else {
		setTime(SECS_SINCE_1900 + (t - regMillis) / 1000);
		fillDataFields();
	}
	return regMillis > 0;
}
