/**
Name:		NTPClass for ESP8266 Arduino IDE
Created:	16.02.2016 23:10:56
Author:		Alexey Bogdan aka Sr.FatCat
ToDo:
*/

#ifndef _NTP_CLIENT_h
#define _NTP_CLIENT_h

#if defined(ARDUINO) && ARDUINO >= 100
#include "arduino.h"
#else
#include "WProgram.h"
#endif

#include <ESP8266WiFi.h>
#include <WiFiUdp.h>

#define OPT_INTERVAL_TIME 60000UL
#define NTP_PACKET_SIZE 48
#define SECS_SINCE_1900 secsSince1970 - 2208988800UL + timeZone * SECS_PER_HOUR
class CNTPTime {
	const uint8_t timeZone;
	const uint16_t ntpPort;  // local port to listen for UDP packets
	WiFiUDP Udp;
	int udpInited;
	unsigned long regMillis;
	unsigned long ntpInterval;
	void getNTPTime();
	inline void fillDataFields();
	unsigned long secsSince1970;

	IPAddress *ntpServers[3];
	char *serversName[3];

public:
	CNTPTime(unsigned long = OPT_INTERVAL_TIME, const uint8_t = 3, const uint16_t = 123, IPAddress * = nullptr, IPAddress * = nullptr, IPAddress * = nullptr);
	CNTPTime(const uint8_t timeZone_) : CNTPTime::CNTPTime(OPT_INTERVAL_TIME, timeZone_) {}
	CNTPTime(const uint16_t ntpPort_) : CNTPTime::CNTPTime(OPT_INTERVAL_TIME, 3, ntpPort_) {}
	CNTPTime(const uint8_t timeZone_, IPAddress *server1, IPAddress *server2 = nullptr, IPAddress *server3 = nullptr)
		: CNTPTime::CNTPTime(OPT_INTERVAL_TIME, timeZone_, 123, server1, server2, server3) {}
	CNTPTime(const uint16_t ntpPort_, IPAddress *server1, IPAddress *server2 = nullptr, IPAddress *server3 = nullptr)
		: CNTPTime::CNTPTime(OPT_INTERVAL_TIME, 3, ntpPort_, server1, server2, server3) {}
	CNTPTime(unsigned long ntpInterval, IPAddress *server1, IPAddress *server2 = nullptr, IPAddress *server3 = nullptr)
		: CNTPTime::CNTPTime(ntpInterval) {}
	CNTPTime(IPAddress *server1, IPAddress *server2 = nullptr, IPAddress *server3 = nullptr)
		: CNTPTime::CNTPTime(OPT_INTERVAL_TIME, 3, 123, server1, server2, server3) {}
	CNTPTime(const uint8_t timeZone_, char *server1, char *server2 = nullptr, char *server3 = nullptr)
		: CNTPTime::CNTPTime(OPT_INTERVAL_TIME, timeZone_) { serversName[0] = server1; serversName[1] = server2; serversName[2] = server3; }
	CNTPTime(const uint16_t ntpPort_, char *server1, char *server2 = nullptr, char *server3 = nullptr)
		: CNTPTime::CNTPTime(OPT_INTERVAL_TIME, 3, ntpPort_) { serversName[0] = server1; serversName[1] = server2; serversName[2] = server3; }
	CNTPTime(unsigned long ntpInterval, char *server1, char *server2 = nullptr, char *server3 = nullptr)
		: CNTPTime::CNTPTime(ntpInterval) { serversName[0] = server1; serversName[1] = server2; serversName[2] = server3; }
	CNTPTime(char *server1, char *server2 = nullptr, char *server3 = nullptr)
		: CNTPTime::CNTPTime() { serversName[0] = server1; serversName[1] = server2; serversName[2] = server3; }
	void Init();
	bool getTime();
	int Year;
	int Month;
	int Day;
	int Hour;
	int Minute;
	int Second;
};
#endif

